import { toWeb3JsMessageFromInput, fromWeb3JsMessage, toWeb3JsTransaction, fromWeb3JsTransaction, toWeb3JsMessage } from '@lorisleiva/js-web3js-adapters';
import { VersionedTransaction, VersionedMessage } from '@solana/web3.js';

class Web3JsTransactionFactory {
  create(input) {
    const web3JsMessage = toWeb3JsMessageFromInput(input);
    return {
      message: fromWeb3JsMessage(web3JsMessage),
      serializedMessage: web3JsMessage.serialize(),
      signatures: input.signatures ?? []
    };
  }
  serialize(transaction) {
    return toWeb3JsTransaction(transaction).serialize();
  }
  deserialize(serializedTransaction) {
    return fromWeb3JsTransaction(VersionedTransaction.deserialize(serializedTransaction));
  }
  serializeMessage(message) {
    return new Uint8Array(toWeb3JsMessage(message).serialize());
  }
  deserializeMessage(serializedMessage) {
    return fromWeb3JsMessage(VersionedMessage.deserialize(serializedMessage));
  }
}

export { Web3JsTransactionFactory };
//# sourceMappingURL=Web3JsTransactionFactory.mjs.map
